unit grafik;                           (* Thomas Klein  1999, 2013 *)

INTERFACE

uses windows,wincrt,graph,dos,printer,maus;

type menu_desc=record                  (* Menue-Beschreibung *)
                typ:(hor0,ver0,rec0,rec1);
                anzahl,anzahl2:word;
                x1,y1,x2,y2,abstand,abstand2:integer;
               end;
var arrow_style:record                 (* Stil fr arrow-Prozedur *)
                 spitze:integer;
                 oeffnung,position:real;
                end;
    numdot_style:record                (* Stil fr numdot-Prozedur *)
                  ausdehnung:integer;
                  groesse:word;
                 end;
    textbox_param:record               (* Textbox-Parameter, (x,y): akt. Pos *)
                   x,y,x1,y1,x2,y2,abstand:integer;
                   groesse,fill,backcol,txtcol:word;
                  end;
    boxoutexit:boolean;                (* true bei boxout-Abbruch *)
    window_param:record                (* Window-Parameter *)
                  x1,y1,x2,y2:integer;
                  xmax,ymax:longint;
                 end;

procedure hireson;
 (* Grafik einschalten *)
function epson(x_oben,y_oben,x_unten,y_unten:integer):word;
 (* S/W-Hardcopy auf Epson-Drucker, Rueckgabe: ioresult *)
function hp(x_oben,y_oben,x_unten,y_unten:integer):word;
 (* S/W-Hardcopy auf HP-Drucker, Rueckgabe: ioresult *)
function hrdcpy(x_oben,y_oben,x_unten,y_unten,typ:integer):word;
 (* Hardcopy, typ: Druckroutine(0=epson, 1=hp), Rueckgabe: ioresult *)
procedure settextnorm(groesse:word);
 (* Groesse/Standardwerte fuer Textstil *)
procedure setlinenorm;
 (* Standardwerte fuer Linienstil *)
procedure box(x1,y1,x2,y2:integer;farbe:word);
 (* ausgefuelltes Rechteck zeichnen *)
procedure outintxy(x,y:integer;num:longint;breite:byte);
 (* num:breite an (x,y) ausgeben *)
procedure outint(num:longint;breite:byte);
 (* num:breite an aktueller Position ausgeben *)
procedure outrealxy(x,y:integer;num:real;breite,nachkomma:byte);
 (* num:breite:nachkomma an (x,y) ausgeben *)
procedure outreal(num:real;breite,nachkomma:byte);
 (* num:breite:nachkomma an aktueller Position ausgeben *)
procedure outstrxy(x,y:integer;s:string;abstand:integer);
 (* string an (x,y) ausgeben, bei '~' Umbruch mit (Zeilen-)abstand *)
procedure outstr(s:string;abstand:integer);
 (* string an akt. Position ausgeben, bei '~' Umbruch mit (Zeilen-)abstand *)
procedure textbar(xoben,yoben,xunten,yunten:integer;s:string;
                  groesse,fill,backcol,txtcol:word);
 (* Balken mit Text, groesse: Textgroesse,
    fill: Fllmuster, bei 0: Rahmen; backcol,txtcol: Farben
    Grafikcursor danach oben hinter Text *)
procedure settextbox(x1,y1,x2,y2,abstand:integer;
                     groesse,fill,backcol,txtcol:word);
 (* Rechteck fr Textausgabe definieren, Parameter s. textbar
    abstand: Zeilenabstand, textheight('*')+2 fr <0 *)
procedure textbox;
 (* Textbox zeichnen('clrscr') *)
procedure boxcr;
 (* Zeilenumbruch in Textbox *)
procedure boxout(s:string;abbruch:boolean);
 (* s in Textbox ausgeben, Zeilenumbruch falls notwendig
    abbruch=true: Abbruch erlaubt, boxoutexit=true bei abbruch *)
function gdir(pfad:string):boolean;
 (* Directory in Textbox ausgeben, false bei Fehler *)
procedure dot(x,y,ausdehnung:integer);
 (* ausgedehnten 'Punkt' zeichnen *)
procedure setnumdotstyle(ausdehnung:integer;groesse:word);
 (* Stil fr numdot: ausdehnung: Punktausdehnung, groesse: Textgroesse *)
procedure setnumdotnorm;
 (* setzt Standardwerte fr numdot *)
procedure numdot(x,y:integer;num:longint);
 (* numerierten Punkt zeichnen *)
procedure setarrowstyle(spitze:integer;oeffnung,position:real);
 (* Stil fr arrow:
    spitze: Spitzengroesse, oeffnung: Oeffnungswinkel in Grad
    position: Spitzenposition, 0: am Anfang - 1: am Ende *)
procedure setarrownorm;
 (* Standardwerte fr arrow-Stil *)
procedure arrow(x1,y1,x2,y2:integer);
 (* Pfeil zeichnen *)
procedure rect(x1,y1,x2,y2:integer;var desc:menu_desc);
 (* rechteckigen Bereich definieren *)
procedure closebutton(x1,y1,x2,y2:integer;backcol,linecol:word;
                      var desc:menu_desc);
 (* zeichnet Schliebutton *)
procedure hmenu(xoben,yoben,xunten,yunten,abstand:integer;menu:string;
                groesse,fill,backcol,txtcol,trenncol:word;var desc:menu_desc);
 (* horizontales Menue,
    abstand: zwischen menuepunkten, <0: abstand=-breite/abstand
    menu: 'p1|p2|...', groesse: schriftfaktor, ...col: farben
    fill: Fuellstil, 0: Rahmen *)
procedure vmenu(xoben,yoben,xunten,yunten,abstand:integer;menu:string;
                groesse,fill,backcol,txtcol,trenncol:word;var desc:menu_desc);
 (* vertikales Menue,
    abstand: zwischen menuepunkten, <0: abstand=-hoehe/abstand
    menu: 'p1|p2|...', groesse: schriftfaktor, ...col: farben
    fill: Fuellstil, 0: Rahmen *)
procedure rmenu(xoben,yoben,xunten,yunten,xabstand,yabstand:integer;
                fill,backcol,trenncol:word;var desc:menu_desc);
 (* rechteckiges Menue,
    xabstand,yabstand: zwischen menuepunkten, <0: abstand=-ausdehnung/abstand
    ...col: Farben, fill: Fuellstil, 0: Rahmen *)
procedure rmenuentry(nr:word;menu:string;
                     groesse,txtcol:word;desc:menu_desc);
 (* menu als Menueeintrag nr in rmenu mit Farbe txtcol und Schrift-groesse *)
function chkrect(x,y,x1,y1,x2,y2:integer):boolean;
 (* x,y in Rechteck? *)
function chkmenu(x,y:integer;desc:menu_desc):word;
 (* (x,y) -> Menuepunkt, 0 fr ausserhalb *)
procedure ginput(var s:string;laenge:byte;backcol,txtcol:word);
 (* Eingabe an aktueller Grafikcursorposition, ...col: Farben *)
procedure setwindow(x1,y1,x2,y2:integer;xmax,ymax:longint);
 (* Fenstergroesse (x1,y1,x2,y2) und Koord.-System (0 bis xmax,0 bis ymax) *)
procedure windowon;
 (* Viewport mit Clipping *)
procedure windowoff;
 (* Viewport=ganzer Bildschirm *)
function xv(x:longint):integer;
 (* neue x-Koord. -> Viewport-y-Koord. *)
function yv(y:longint):integer;
 (* neue y-Koord. -> Viewport-y-Koord. *)
function vx(x:integer):longint;
 (* Viewport-x-Koord. -> neue x-Koord. *)
function vy(y:integer):longint;
 (* Viewport-y-Koord. -> neue y-Koord. *)
function mx(x:integer):longint;
 (* Maus-x-Koord. -> neue x-Koord. *)
function my(y:integer):longint;
 (* Maus-y-Koord. -> neue y-Koord. *)
function xm(x:longint):integer;
 (* neue x-Koord. -> Maus-x-Koord. *)
function ym(y:longint):integer;
 (* neue y-Koord. -> Maus-y-Koord. *)

IMPLEMENTATION

procedure hireson;
 (* Grafik einschalten *)
 var graphdriver,graphmode:integer;
 begin
  graphdriver:=D8Bit; graphmode:=m800x600;
  initgraph(graphdriver,graphmode,'');
  if graphresult<>0 then begin
   graphdriver:=D8Bit; graphmode:=m640x480;
   initgraph(graphdriver,graphmode,'');
  end;
  if graphresult<>0 then begin
   graphdriver:=detect; graphmode:=0;
   initgraph(graphdriver,graphmode,'');
  end;
 end;

function epson(x_oben,y_oben,x_unten,y_unten:integer):word;
 (* S/W-Hardcopy auf Epson-Drucker, Rueckgabe: ioresult *)
 const esc=#27;
 var line:array [1..720] of byte;
     modus:byte;
     i,x,y,breite,hoehe:integer;
 procedure printline;
  var i:integer;
  begin
{$I-}
   write(lst,esc,'*',chr(modus),chr(lo(breite)),chr(hi(breite)));
   for i:=1 to breite do write(lst,chr(line[i]));
   write(lst,#13,#10);
   fillchar(line,breite,0);
  end;
 begin
  modus:=6;
  if (getmaxx=319) or ((getmaxx=639) and (getmaxy=479)) then modus:=4;
  if (getmaxx=639) and (getmaxy=199) then modus:=1;
  i:=ioresult; write(lst,esc,'@'); (* Initialisierung *)
  i:=ioresult;
  if i=0 then begin
   write(lst,esc,'A',#8); (* 8/60 Zoll-Vorschub *)
   breite:=x_unten-x_oben+1; hoehe:=y_unten-y_oben+1;
   fillchar(line,breite,0);
   for y:=1 to hoehe do begin
    for x:=1 to breite do
     if getpixel(x_oben+x-1,y_oben+y-1)<>0 then inc(line[x],1 shl (7-(y-1) mod 8));
    if y mod 8=0 then printline;
   end;
   printline; write(lst,esc,'@');
{I+}
  end;
  epson:=i;
 end;
function hp(x_oben,y_oben,x_unten,y_unten:integer):word;
 (* S/W-Hardcopy auf HP-Drucker, Rueckgabe: ioresult *)
 const esc=#27;
 var i,x,y,b,breite:word;
     p,d:byte;
 begin
{$I-}
  i:=ioresult; write(lst,esc,'*t100R',esc,'*r1A'); i:=ioresult;
  if i=0 then begin
   breite:=(x_unten-x_oben+1) div 8;
   if (x_unten-x_oben+1) mod 8<>0 then inc(breite);
   for y:=y_oben to y_unten do begin
    write(lst,esc,'*b',breite:1,'W'); x:=x_oben;
    for b:=1 to breite do begin
     d:=0;
     for p:=0 to 7 do begin
      if (x<=x_unten) and (getpixel(x,y)<>0) then inc(d,1 shl(7-p));
      inc(x);
     end;
     write(lst,chr(d));
    end;
   end;
   writeln(lst,esc,'*rB');
  end;
{$I+}
  hp:=i;
 end;
function hrdcpy(x_oben,y_oben,x_unten,y_unten,typ:integer):word;
 (* Hardcopy, typ: Druckroutine(0=epson, 1=hp), Rueckgabe: ioresult *)
 begin
  case typ of
   0: hrdcpy:=epson(x_oben,y_oben,x_unten,y_unten);
   1: hrdcpy:=hp(x_oben,y_oben,x_unten,y_unten);
  end;
 end;

procedure settextnorm(groesse:word);
 (* Groesse/Standardwerte fuer Textstil *)
 begin
  settextstyle(0,horizdir,groesse); settextjustify(lefttext,toptext);
 end;
procedure setlinenorm;
 (* Standardwerte fuer Linienstil *)
 begin
  setlinestyle(solidln,0,1);
 end;

procedure box(x1,y1,x2,y2:integer;farbe:word);
 (* ausgefuelltes Rechteck zeichnen *)
 begin
  setfillstyle(solidfill,farbe); bar(x1,y1,x2,y2);
 end;

procedure outintxy(x,y:integer;num:longint;breite:byte);
 (* num:breite an (x,y) ausgeben *)
 var s:string;
 begin
  str(num:breite,s); outtextxy(x,y,s);
 end;
procedure outint(num:longint;breite:byte);
 (* num:breite an aktueller Position ausgeben *)
 var s:string;
 begin
  str(num:breite,s); outtext(s);
 end;
procedure outrealxy(x,y:integer;num:real;breite,nachkomma:byte);
 (* num:breite:nachkomma an (x,y) ausgeben *)
 var s:string;
 begin
  str(num:breite:nachkomma,s); outtextxy(x,y,s);
 end;
procedure outreal(num:real;breite,nachkomma:byte);
 (* num:breite:nachkomma an aktueller Position ausgeben *)
 var s:string;
 begin
  str(num:breite:nachkomma,s); outtext(s);
 end;
procedure outstrxy(x,y:integer;s:string;abstand:integer);
 (* string an (x,y) ausgeben, bei '~' Umbruch mit (Zeilen-)abstand *)
 var p:byte;
 begin
  while length(s)>0 do begin
   p:=pos('~',s); if p=0 then p:=length(s)+1;
   outtextxy(x,y,copy(s,1,p-1));
   delete(s,1,p); inc(y,abstand);
  end;
 end;
procedure outstr(s:string;abstand:integer);
 (* string an akt. Position ausgeben, bei '~' Umbruch mit (Zeilen-)abstand *)
 var x,y,xmax:integer;
     p:byte;
 begin
  x:=getx; y:=gety; xmax:=x;
  while length(s)>0 do begin
   p:=pos('~',s); if p=0 then p:=length(s)+1;
   outtext(copy(s,1,p-1)); if getx>xmax then xmax:=getx;
   delete(s,1,p); moveto(x,gety+abstand);
  end;
  moveto(xmax,y);
 end;
procedure textbar(xoben,yoben,xunten,yunten:integer;s:string;
                  groesse,fill,backcol,txtcol:word);
 (* Balken mit Text, groesse: Textgroesse,
    fill: Fllmuster, bei 0: Rahmen; backcol,txtcol: Farben
    Grafikcursor danach oben hinter Text *)
 begin
  setfillstyle(fill,backcol); bar(xoben,yoben,xunten,yunten);
  if fill=emptyfill then begin
   setlinenorm; setcolor(backcol); rectangle(xoben,yoben,xunten,yunten);
  end;
  settextnorm(groesse); setcolor(txtcol);
  moveto(xoben+4,yoben+3); outstr(s,textheight('*')+2);
 end;

procedure settextbox(x1,y1,x2,y2,abstand:integer;
                     groesse,fill,backcol,txtcol:word);
 (* Rechteck fr Textausgabe definieren, Parameter s. textbar
    abstand: Zeilenabstand, textheight('*')+2 fr <0 *)
 begin
  settextnorm(groesse);
  textbox_param.x1:=x1; textbox_param.y1:=y1;
  textbox_param.x2:=x2; textbox_param.y2:=y2;
  if abstand<0 then
   textbox_param.abstand:=textheight('*')+2
  else
   textbox_param.abstand:=abstand;
  textbox_param.groesse:=groesse; textbox_param.fill:=fill;
  textbox_param.backcol:=backcol; textbox_param.txtcol:=txtcol;
  textbox_param.x:=x1; textbox_param.y:=maxint;
   (* maxint: textbox noch nicht gezeichnet *)
 end;
procedure textbox;
 (* Textbox zeichnen('clrscr') *)
 begin
  with textbox_param do begin
   setfillstyle(fill,backcol); bar(x1,y1,x2,y2);
   if fill=emptyfill then begin
    setlinenorm; setcolor(backcol); rectangle(x1,y1,x2,y2);
   end;
   x:=x1+4; y:=y1+3;
  end;
 end;
procedure boxcr;
 (* Zeilenumbruch in Textbox *)
 begin
  with textbox_param do begin
   x:=x1+4; inc(y,abstand);
  end;
 end;
procedure boxout(s:string;abbruch:boolean);
 (* s in Textbox ausgeben, Zeilenumbruch falls notwendig
    abbruch=true: Abbruch erlaubt, boxoutexit=true bei abbruch *)
 var c:char;
     width:word;
 begin
  with textbox_param do begin
   width:=textwidth(s);
   if x+width>=x2 then boxcr;
   if y+textheight(s)>=y2 then begin
    if y<>maxint then begin
     if mausok then begin warte_maus(0); zeige_maus; end;
     repeat
      if mausok then mausstatus else maus_status:=0;
     until (maus_status>0) or keypressed;
     if mausok then verstecke_maus;
     if keypressed then c:=readkey else c:=' ';
     if abbruch and ((maus_status>1) or (upcase(c) in ['Q','X','E','A'])) then begin
      boxoutexit:=true; exit;
     end;
    end;
    textbox;
   end;
   settextnorm(groesse); setcolor(txtcol);
   outtextxy(x,y,s); inc(x,width);
  end;
  boxoutexit:=false;
 end;
function gdir(pfad:string):boolean;
 (* Directory in Textbox ausgeben, false bei Fehler *)
 var eintrag:searchrec;
     name:string;
     d:integer;
 begin
  findfirst(pfad,directory+readonly,eintrag); d:=doserror;
  while d=0 do begin
   name:=eintrag.name;
   if pos('.',name+'.')>9 then begin
    delete(name,9,pos('.',name+'.')-9); name[8]:='~';
   end;
   while pos('.',name+'.')<9 do
    insert(' ',name,pos('.',name+'.'));
   while length(name)<12 do
    name:=name+' ';
   if length(name)>12 then name:=copy(name,1,11)+'~';
   boxout(name+'  ',true);
   if boxoutexit then
    d:=18
   else begin
    findnext(eintrag); d:=doserror;
   end;
  end;
  gdir:=d=18;
 end;

procedure dot(x,y,ausdehnung:integer);
 (* ausgedehnten 'Punkt' zeichnen *)
 begin
  box(x-ausdehnung,y-ausdehnung,x+ausdehnung,y+ausdehnung,getcolor);
 end;
procedure setnumdotstyle(ausdehnung:integer;groesse:word);
 (* Stil fr numdot: ausdehnung: Punktausdehnung, groesse: Textgroesse *)
 begin
  numdot_style.ausdehnung:=ausdehnung;
  numdot_style.groesse:=groesse;
 end;
procedure setnumdotnorm;
 (* setzt Standardwerte fr numdot *)
 begin
  setnumdotstyle(1,1);
 end;
procedure numdot(x,y:integer;num:longint);
 (* numerierten Punkt zeichnen *)
 begin
  with numdot_style do begin
   box(x-ausdehnung,y-ausdehnung,x+ausdehnung,y+ausdehnung,getcolor);
   settextnorm(groesse); outintxy(x+ausdehnung+2,y+ausdehnung+2,num,1);
  end;
 end;

procedure setarrowstyle(spitze:integer;oeffnung,position:real);
 (* Stil fr arrow:
    spitze: Spitzengroesse, oeffnung: Oeffnungswinkel in Grad
    position: Spitzenposition, 0: am Anfang - 1: am Ende *)
 begin
  arrow_style.spitze:=spitze;
  arrow_style.oeffnung:=oeffnung*pi/180;
  arrow_style.position:=position;
 end;
procedure setarrownorm;
 (* Standardwerte fr arrow-Stil *)
 begin
  setarrowstyle(8,20,1);
 end;
procedure arrow(x1,y1,x2,y2:integer);
 (* Pfeil zeichnen *)
 var x,y:integer;
     w,w1:real;
 begin
  if x1<>x2 then begin
   w:=-arctan((y2-y1)/(x2-x1));
   if x1>x2 then w:=w+pi;
  end else
   if y1>y2 then w:=pi/2 else w:=-pi/2;
  setlinenorm;
  if arrow_style.position<>1 then begin
   x:=x2; y:=y2;
   x2:=x1+round((x2-x1)*arrow_style.position);
   y2:=y1+round((y2-y1)*arrow_style.position);
  end;
  line(x1,y1,x2,y2);
  w1:=pi/2-w-arrow_style.oeffnung;
  line(x2,y2,round(x2-arrow_style.spitze*sin(w1)),round(y2+arrow_style.spitze*cos(w1)));
  w1:=w-arrow_style.oeffnung;
  line(x2,y2,round(x2-arrow_style.spitze*cos(w1)),round(y2+arrow_style.spitze*sin(w1)));
  if arrow_style.position<>1 then line(x2,y2,x,y);
 end;

procedure rect(x1,y1,x2,y2:integer;var desc:menu_desc);
 (* rechteckigen Bereich definieren *)
 begin
  desc.typ:=rec0; desc.anzahl:=1;
  desc.x1:=x1; desc.y1:=y1; desc.x2:=x2; desc.y2:=y2;
 end;
procedure closebutton(x1,y1,x2,y2:integer;backcol,linecol:word;
                      var desc:menu_desc);
 (* zeichnet Schliebutton *)
 begin
  box(x1+1,y1+1,x2-1,y2-1,backcol);
  setlinenorm; setcolor(linecol); rectangle(x1,y1,x2,y2);
  line(x1,y1,x2,y2); line(x1,y2,x2,y1);
  rect(x1,y1,x2,y2,desc);
 end;
procedure hmenu(xoben,yoben,xunten,yunten,abstand:integer;menu:string;
                groesse,fill,backcol,txtcol,trenncol:word;var desc:menu_desc);
 (* horizontales Menue,
    abstand: zwischen menuepunkten, <0: abstand=-breite/abstand
    menu: 'p1|p2|...', groesse: schriftfaktor, ...col: Farben
    fill: Fuellstil, 0: Rahmen *)
 var p:byte;
 begin
  if abstand<0 then abstand:=(xoben-xunten-1) div abstand;
  desc.typ:=hor0; desc.abstand:=abstand;
  desc.x1:=xoben; desc.y1:=yoben; desc.x2:=xunten; desc.y2:=yunten;
  setfillstyle(fill,backcol); bar(xoben,yoben,xunten,yunten);
  setlinenorm;
  if fill=emptyfill then begin
   setcolor(backcol); rectangle(xoben,yoben,xunten,yunten);
  end;
  settextnorm(groesse);
  desc.anzahl:=0;
  while length(menu)>0 do begin
   p:=pos('|',menu); if p=0 then p:=length(menu)+1;
   setcolor(trenncol);
   if desc.anzahl>0 then
    if fill=emptyfill then
     line(xoben,yoben+1,xoben,yunten-1)
    else
     line(xoben,yoben,xoben,yunten);
   setcolor(txtcol);
   outstrxy(xoben+4,yoben+3,copy(menu,1,p-1),textheight('*')+2);
   delete(menu,1,p); inc(xoben,abstand); inc(desc.anzahl);
  end;
 end;
procedure vmenu(xoben,yoben,xunten,yunten,abstand:integer;menu:string;
                groesse,fill,backcol,txtcol,trenncol:word;var desc:menu_desc);
 (* vertikales Menue,
    abstand: zwischen menuepunkten, <0: abstand=-hoehe/abstand
    menu: 'p1|p2|...', groesse: schriftfaktor, ...col: Farben
    fill: Fuellstil, 0: Rahmen *)
 var p:byte;
 begin
  if abstand<0 then abstand:=(yoben-yunten-1) div abstand;
  desc.typ:=ver0; desc.abstand:=abstand;
  desc.x1:=xoben; desc.y1:=yoben; desc.x2:=xunten; desc.y2:=yunten;
  setfillstyle(fill,backcol); bar(xoben,yoben,xunten,yunten);
  setlinenorm;
  if fill=emptyfill then begin
   setcolor(backcol); rectangle(xoben,yoben,xunten,yunten);
  end;
  settextnorm(groesse);
  desc.anzahl:=0;
  while length(menu)>0 do begin
   p:=pos('|',menu); if p=0 then p:=length(menu)+1;
   setcolor(trenncol);
   if desc.anzahl>0 then
    if fill=emptyfill then
     line(xoben+1,yoben,xunten-1,yoben)
    else
     line(xoben,yoben,xunten,yoben);
   setcolor(txtcol);
   outstrxy(xoben+3,yoben+3,copy(menu,1,p-1),textheight('*')+2);
   delete(menu,1,p); inc(yoben,abstand); inc(desc.anzahl);
  end;
 end;
procedure rmenu(xoben,yoben,xunten,yunten,xabstand,yabstand:integer;
                fill,backcol,trenncol:word;var desc:menu_desc);
 (* rechteckiges Menue,
    xabstand,yabstand: zwischen menuepunkten, <0: abstand=-ausdehnung/abstand
    ...col: Farben, fill: Fuellstil, 0: Rahmen *)
 var i,p:integer;
 begin
  if xabstand<0 then xabstand:=(xoben-xunten-1) div xabstand;
  if yabstand<0 then yabstand:=(yoben-yunten-1) div yabstand;
  desc.typ:=rec1; desc.abstand:=xabstand; desc.abstand2:=yabstand;
  desc.x1:=xoben; desc.y1:=yoben; desc.x2:=xunten; desc.y2:=yunten;
  setfillstyle(fill,backcol); bar(xoben,yoben,xunten,yunten);
  setlinenorm;
  if fill=emptyfill then begin
   setcolor(backcol); rectangle(xoben,yoben,xunten,yunten);
  end;
  setcolor(trenncol);
  desc.anzahl2:=(xunten-xoben+1) div xabstand; p:=xoben;
  for i:=2 to desc.anzahl2 do begin
   inc(p,xabstand);
   if fill=emptyfill then
    line(p,yoben+1,p,yunten-1)
   else
    line(p,yoben,p,yunten);
  end;
  desc.anzahl:=(yunten-yoben+1) div yabstand; p:=yoben;
  for i:=2 to desc.anzahl do begin
   inc(p,yabstand);
   if fill=emptyfill then
    line(xoben+1,p,xunten-1,p)
   else
    line(xoben,p,xunten,p);
  end;
  desc.anzahl:=desc.anzahl*desc.anzahl2;
 end;
procedure rmenuentry(nr:word;menu:string;
                     groesse,txtcol:word;desc:menu_desc);
 (* menu als Menueeintrag nr in rmenu mit Farbe txtcol und Schrift-groesse *)
 begin
  dec(nr); settextnorm(groesse); setcolor(txtcol);
  with desc do
   outstrxy(x1+(nr mod anzahl2)*abstand+4,y1+(nr div anzahl2)*abstand2+3,
            menu,textheight('*')+2);
 end;
function chkrect(x,y,x1,y1,x2,y2:integer):boolean;
 (* x,y in Rechteck? *)
 begin
  chkrect:=(x>=x1) and (x<=x2) and (y>=y1) and (y<=y2);
 end;
function chkmenu(x,y:integer;desc:menu_desc):word;
 (* (x,y) -> Menuepunkt, 0 fr ausserhalb *)
 var punkt:word;
 begin
  if chkrect(x,y,desc.x1,desc.y1,desc.x2,desc.y2) then begin
   case desc.typ of
    hor0: punkt:=(x-desc.x1) div desc.abstand+1;
    ver0: punkt:=(y-desc.y1) div desc.abstand+1;
    rec0: punkt:=1;
    rec1: begin
           punkt:=(x-desc.x1) div desc.abstand+1;
           if punkt>desc.anzahl2 then punkt:=desc.anzahl2;
           inc(punkt,(y-desc.y1) div desc.abstand2*desc.anzahl2);
           if punkt>desc.anzahl then dec(punkt,desc.anzahl2);
          end;
   end;
   if punkt>desc.anzahl then punkt:=desc.anzahl;
  end else
   punkt:=0;
  chkmenu:=punkt;
 end;

procedure ginput(var s:string;laenge:byte;backcol,txtcol:word);
 (* Eingabe an aktueller Grafikcursorposition, ...col: Farben *)
 var c,d:char;
     x,y,width,height:integer;
     x1:byte;
 begin
  x1:=length(s); x:=getx; y:=gety; setcolor(txtcol);
  width:=textwidth('*'); height:=textheight('*'); setlinenorm;
  repeat
   box(x,y,x+laenge*width-1,y+height+1,backcol);
   outtextxy(x,y,s);
   if x1>=laenge then x1:=laenge-1;
   line(x+x1*width,y+height+1,x+x1*width+width-1,y+height+1);
   c:=readkey; if c=#0 then d:=readkey;
   if c=#8 then begin d:=c; c:=#0; end;
   if c=#0 then
    case ord(d) of
     77 : if x1<length(s) then inc(x1); (* rechts *)
     75 : if x1>0 then dec(x1); (* links *)
      8 : if x1>0 then begin (* backspace *)
                        delete(s,x1,1); dec(x1);
                       end;
     82 : if (length(s)<laenge) and (x1<length(s)) then
           insert(#32,s,x1+1); (* insert *)
     83 : delete(s,x1+1,1); (* delete *)
     71 : x1:=0; (* home *)
     79 : x1:=length(s); (* end *)
    end
   else
    if (c>#31) and (length(s)<laenge) then begin
     insert(c,s,x1+1); inc(x1);
    end;
  until c=#13;
 end;

procedure setwindow(x1,y1,x2,y2:integer;xmax,ymax:longint);
 (* Fenstergroesse (x1,y1,x2,y2) und Koord.-System (0 bis xmax,0 bis ymax) *)
 begin
  window_param.x1:=x1; window_param.y1:=y1;
  window_param.x2:=x2; window_param.y2:=y2;
  window_param.xmax:=xmax; window_param.ymax:=ymax;
 end;
procedure windowon;
 (* Viewport mit Clipping *)
 begin
  with window_param do
   setviewport(x1,y1,x2,y2,true);
 end;
procedure windowoff;
 (* Viewport=ganzer Bildschirm *)
 begin
  setviewport(0,0,getmaxx,getmaxy,true);
 end;
function xv(x:longint):integer;
 (* neue x-Koord. -> Viewport-y-Koord. *)
 begin
  with window_param do
   xv:=x*(x2-x1) div xmax;
 end;
function yv(y:longint):integer;
 (* neue y-Koord. -> Viewport-y-Koord. *)
 begin
  with window_param do
   yv:=y*(y2-y1) div ymax;
 end;
function vx(x:integer):longint;
 (* Viewport-x-Koord. -> neue x-Koord. *)
 begin
  with window_param do
   vx:=x*xmax div (x2-x1);
 end;
function vy(y:integer):longint;
 (* Viewport-y-Koord. -> neue y-Koord. *)
 begin
  with window_param do
   vy:=y*ymax div (y2-y1);
 end;
function mx(x:integer):longint;
 (* Maus-x-Koord. -> neue x-Koord. *)
 begin
  mx:=vx(x-window_param.x1);
 end;
function my(y:integer):longint;
 (* Maus-y-Koord. -> neue y-Koord. *)
 begin
  my:=vy(y-window_param.y1);
 end;
function xm(x:longint):integer;
 (* neue x-Koord. -> Maus-x-Koord. *)
 begin
  xm:=xv(x)+window_param.x1;
 end;
function ym(y:longint):integer;
 (* neue y-Koord. -> Maus-y-Koord. *)
 begin
  ym:=yv(y)+window_param.y1;
 end;


begin
 setnumdotnorm; setarrownorm;
end.

