unit tsp;                           (* Thomas Klein 1999, FreePascal: 2013 *)

INTERFACE

uses sysutils,dateutils,graph,printer,dos,grafik,maus;

const version=4.01;                 (* Versionsnummer *)
      maxstadt=75;                  (* max. Anzahl v. Staedten *)
      maxstadtquad=5625;            (* maxstadt^2 *)
      maxentfernung=maxlongint;     (* max. Entfernung *)
      maxstrecke=28633115;          (* max. Entfernung(+1) zw. zwei Staedten,
                                       maxlongint/maxstadt *)
      maxx=1240; maxy=999;          (* max. Koord. f. Karte *)
      maxzufall=1599;               (* max. Entfernung(/1000) bei zufaelliger
                                       Erstellung v. entftab *)
      maxverfahren=40;              (* max. Anzahl v. Loesungsverfahren *)

type stadt=record                   (* Stadtkoord. in Karte *)
            x,y:longint;
           end;
     stadtnr=0..maxstadt;           (* Nr. von Staedten *)
     entfernung=longint;            (* Entfernungen zw. Staedten *)
     entfmatrix=array [1..maxstadt,1..maxstadt] of entfernung;
                                    (* Entfernungsmatrix *)
     entftyp=(kart,symm,asymm);     (* Art der Entfernungsbestimmung
                                       kart: aus Karte, symm: zufaellig
                                       symmetrisch, asymm: zufaellig asymm. *)
     weg=string;                    (* Wege werden in Strings gespeichert *)
     wegtypmenge=set of byte;       (* Byte-Menge fuer Wegtypen,
                                       s. function wegtyp *)

var stadtkarte:array [1..maxstadt] of stadt;   (* Koord. der Staedte *)
    entftab:entfmatrix;             (* Entfernungstabelle *)
    entftabtyp:entftyp;             (* Art der Entfernungsbestimmung *)
    vorher,aktuell:weg;             (* vorheriger u. aktueller Weg *)
    stadtanzahl:stadtnr;            (* Anzahl Staedte *)
    verfahren:byte;                 (* ausgewaehltes Loesungsverfahren *)
    verfanzahl:byte;                (* Anzahl Loesungsverfahren *)
    verfwegtypen:array [1..maxverfahren] of wegtypmenge;
                                    (* Wegtypmengen der Verfahren *)
    verftext:array [1..maxverfahren] of string[100]; (* Menuetexte/Titel d.
                                       Verf., s. procedure verfahren_waehlen *)
    verfformat:record               (* Format des Menues zur Verfahrenswahl *)
                x,y:byte;           (* x: Spalten, y: Zeilen *)
               end;
    startpunktwahl:boolean;         (* true: automatische Startpunktwahl *)
    oben,unten,rechts:integer;      (* Position der Trennlinien zw. Menues und
                                       Karte, links=0 *)
    leiste,button,karte:menu_desc;  (* Menue-Descriptoren *)

procedure wegtxt(w:weg;warten:boolean);
 (* Weg w in Textmodus ausgeben, evtl. nach Ausgabe warten *)

procedure neustart;
 (* setzt wichtige Variablen zurueck *)

function weglaenge(w:weg):entfernung;
 (* ermittelt Weglaenge, 0 falls keine Entfernungstab. vorhanden *)
function wegtyp(w:weg):byte;
 (* ermittelt 'Weg'-Typ; 0: kein Weg, 1: Weg vorhanden,
    2: keine doppelten Staedte, 3: wie 2 und zurueck zum Start;
    +10: keine Unterbrechung, +100: alle Staedte *)

procedure menue(abstand:integer;menu:string;
          backcol,txtcol,trenncol:word;var desc:menu_desc);
 (* zeichnet Menue in untere Leiste, Parameter wie bei hmenu *)
procedure meldung(s:string;hintergrund,schrift:word);
 (* Meldung in unterer Leiste *)
procedure laenge_aus(w:weg);
 (* gibt Weglnge in unterer Leiste aus *)
procedure wegtyp_aus(w:weg);
 (* gibt Wegtyp in unterer Leiste aus *)
procedure bildschirm;
 (* zeichnet Rahmen und rechtes Menue *)

procedure start0;
 (* startet Stoppuhr *)
procedure start;
 (* 'Loesung wird berechnet'(und evtl. Serienteil) ausgeben und
     Stoppuhr starten *)
procedure stop;
 (* stoppt Stoppuhr *)
function zeit:real;
 (* liefert gestoppte Zeit in Sek. *)
procedure zeit_aus(komma:boolean);
 (* gibt gestoppte Zeit an akt. Pos. aus, evtl. mit vorangestelltem Komma *)

procedure zeichne_stadt(nr:stadtnr);
 (* zeichnet Stadt nr in Karte ein, vorher windowon noetig *)
procedure zeichne_kante(stadt1,stadt2:stadtnr);
 (* zeichnet Pfeil v. stadt1 nach stadt2, vorher window noetig *)
procedure zeichne_karte(loeschen:boolean;farbe:word);
 (* zeichnet Staedte mit farbe in Karte, loeschen=true: bildschirm loeschen *)
procedure zeichne_weg(w:weg;loeschen:boolean;farbe:word);
 (* zeichnet Weg mit farbe in Karte, loeschen=true: bildschirm loeschen *)
procedure farbstadt(nr:stadtnr;farbe:word);
 (* zeichnet Stadt nr mit farbe in Karte *)
procedure startstadt(w:weg;farbe:word);
 (* zeichnet Startstadt von w mit farbe in Karte *)
procedure wegkarte(w:weg;loeschen:boolean;wegfarbe,kartenfarbe:word);
 (* zeichnet Weg und Karte, loeschen=true: bildschirm loeschen *)

procedure bestimme_entfernung(nr:stadtnr;typ:entftyp);
 (* traegt Entfernungen fuer Stadt nr in entftab ein, Berechnung
    entsprechend typ *)
procedure zeige_entftab(schirm:boolean);
 (* gibt entftab auf Bildschirm(true) oder in Datei(false) aus *)

function finde_stadt(xx,yy:longint):stadtnr;
 (* findet zu (xx,yy) naechste Stadt *)
function stadtwahl(button:byte):stadtnr;
 (* wartet auf 'button'(255: bel.) und bestimmt naechstgelegene Stadt *)
procedure neue_stadt(xx,yy:longint);
 (* traegt stadt mit Koord. (xx,yy) in stadtkarte ein und zeichnet sie *)
procedure loesche_stadt(nr:stadtnr);
 (* entfernt Stadt nr aus stadtkarte, Wege werden geloescht *)

procedure nrweg(var w:weg);
 (* erzeugt Rundweg nach Staedtnr. *)
procedure zufallsweg(var w:weg);
 (* erzeugt zufaelligen Rundweg *)
procedure manuellweg(var w:weg);
 (* manuelle Wegeingabe *)

function dateiname(prompt:string):string;
 (* liefert Dateiname, bei $ wird Directory angezeigt *)
procedure speichern(name:string);
 (* als ASCII-Datei, kann mit bel. Editor bearbeitet werden *)
procedure laden(name:string);
 (* Nichtzahlen, Zeilenenden und mit '#' beginnende Zeilen werden ueberlesen *)
function serienteil(wegtypen:wegtypmenge):boolean;
 (* letzte Serienberechnung speichern, naechste laden,
    false: keine weitere Datei oder falscher Wegtyp *)

procedure eine_stadt;
 (* Mausklick auf Karte -> eine Stadt ergaenzen/loeschen *)
procedure neu;
 (* alles loeschen *)
procedure staedte_ergaenzen;
 (* mehrere Staedte ergaenzen *)
procedure staedte_loeschen;
 (* mehrere Staedte loeschen *)
procedure entfernungen;
 (* Entfernungen berechnen/eingeben *)
procedure weg_bestimmen;
 (* auf verschiedene Arten Weg vorgeben *)
procedure verfahren_waehlen(var wegtypen:wegtypmenge);
 (* verfahren:=nr Lsungsverfahren, wegtypen:=erlaubte Wegtypen des Verf.
    Format verftext: '*Menueeintrag|Titelzeile',
    *=+: fragen, ob automatische Startpunktwahl gewuenscht *)
function loesen(wegtypen:wegtypmenge):boolean;
 (* true: wegtyp(aktuell) in Menge wegtypen enthalten und entftab
    vorhanden, sonst Meldung *)
procedure serie;
 (* Load-/Save-Name fuer Serienberechnung *)
// procedure drucken;
// (* Karte und Weglaenge drucken *)
procedure datei_speichern;
 (* Name waehlen und speichern *)
procedure datei_laden;
 (* Name waehlen und laden *)
function ende:boolean;
 (* Sicherheitsabfrage *)

function tspmenue(wegtypen:wegtypmenge):boolean;
 (* ueberwacht Menue, Verlassen mit Ende/Schliessbutton(false) oder
    Loesen(true, nur wenn aktueller Weg in wegtypen-Menge *)
function tspinit(titel,aufruf:string):boolean;
 (* Initialisierungen, Bildschirm zeichnen
    titel: Fenstertitel, aufruf: Menuetext Loesungsroutine
    false: kein Maustreiber installiert *)

IMPLEMENTATION

var entftabok:boolean;              (* true wenn Entfernungstab. vorhanden *)
    tsp_titel,tsp_aufruf:string;    (* Fenster-/Menuetext *)
    tsp_wegtypen:wegtypmenge;       (* erlaubte Wegtypen *)
    zeit1,zeit2:tdatetime;          (* Zeiten fuer Stoppuhr *)
    menue1,menue2:menu_desc;        (* menue1: Hauptmenue, menue2: unten *)
    letzte_meldung:boolean;         (* true: letzte meldung war weglaenge *)
    datei:text;                     (* Dateivar. fuer Speichern/Laden *)
    lname,sname:string;             (* Dateinamen fuer Serie *)
    vname:char;                     (* var. Teil der Seriennamen *)

procedure wegtxt(w:weg;warten:boolean);
 (* Weg w in Textmodus ausgeben, evtl. nach Ausgabe warten *)
 var i:byte;
 begin
  for i:=1 to length(w) do write(ord(w[i]):4);
  writeln;
  if warten then warte;
 end;

procedure neustart;
 (* setzt wichtige Variablen zurueck *)
 begin
  zeit1:=0; entftabok:=false;
  stadtanzahl:=0; vorher:=''; aktuell:='';
 end;

function weglaenge(w:weg):entfernung;
 (* ermittelt Weglaenge, 0 falls keine Entfernungstab. vorhanden *)
 var laenge:entfernung;
     i:byte;
 begin
  laenge:=0;
  for i:=2 to length(w) do
   if (ord(w[i-1])<>0) and (ord(w[i])<>0) then
     inc(laenge,entftab[ord(w[i-1]),ord(w[i])]);
  weglaenge:=laenge;
 end;
function wegtyp(w:weg):byte;
 (* ermittelt 'Weg'-Typ; 0: kein Weg, 1: Weg vorhanden,
    2: keine doppelten Staedte, 3: wie 2 und zurueck zum Start;
    +10: keine Unterbrechung, +100: alle Staedte *)
 var i,l,typ:byte;
     staedte:set of 0..maxstadt;
 begin
  staedte:=[];
  if w='' then
   typ:=0
  else begin
   typ:=2; i:=1; l:=length(w);
   while (i<=l) and (typ=2) do
    if ord(w[i]) in staedte then
     typ:=1
    else begin
     staedte:=staedte+[ord(w[i])]; inc(i);
    end;
   if (typ=1) and (i=l) and (w[i]=w[1]) then typ:=3;
   if not(0 in staedte) then inc(typ,10);
   while i<=l do begin
    staedte:=staedte+[ord(w[i])]; inc(i);
   end;
   if (staedte=[1..stadtanzahl]) or (staedte=[0..stadtanzahl]) then
    inc(typ,100);
  end;
  wegtyp:=typ;
 end;

procedure menue(abstand:integer;menu:string;
          backcol,txtcol,trenncol:word;var desc:menu_desc);
 (* zeichnet Menue in untere Leiste, Parameter wie bei hmenu *)
 begin
  verstecke_maus;
  hmenu(1,unten+1,getmaxx-1,getmaxy-1,abstand,menu,1,1,
        backcol,txtcol,trenncol,desc);
  zeige_maus;
 end;
procedure meldung(s:string;hintergrund,schrift:word);
 (* Meldung in unterer Leiste *)
 begin
  verstecke_maus;
  textbar(1,unten+1,getmaxx-1,getmaxy-1,s,1,1,hintergrund,schrift);
  zeige_maus; letzte_meldung:=false;
 end;
procedure laenge_aus(w:weg);
 (* gibt Weglnge in unterer Leiste aus *)
 var s:string;
 begin
  if entftabok then begin
   str(weglaenge(w)/1000:1:3,s);
   meldung('Lnge des dargestellten Weges: '+s,blue,white);
  end else
   meldung('Keine Entfernungstabelle vorhanden',blue,white);
  letzte_meldung:=true;
 end;
procedure wegtyp_aus(w:weg);
 (* gibt Wegtyp in unterer Leiste aus *)
 begin
  verstecke_maus;
  meldung('Typ des dargestellten Weges: ',blue,white); outint(wegtyp(w),1);
  zeige_maus;
 end;
procedure bildschirm;
 (* zeichnet Rahmen und rechtes Menue *)
 begin
  setlinenorm; setcolor(lightgray);
  rectangle(0,0,getmaxx,getmaxy);
  rectangle(rechts,0,getmaxx,unten);
  line(0,unten,getmaxx,unten);
  line(0,oben,getmaxx,oben);
  textbar(1,1,getmaxx-1,oben-1,tsp_titel,1,1,blue,white);
  closebutton(getmaxx-14,2,getmaxx-4,12,lightgray,black,button);
  if verfanzahl=0 then
   vmenu(rechts+1,oben+1,getmaxx-1,unten-1,-10, // -11
         'Neu|Stdte~ergnzen|Stdte~lschen|Entfer-~nungen|Weg|'
//         +tsp_aufruf+'|Serie|Drucken|Speichern|Laden|Ende',
         +tsp_aufruf+'|Serie|Speichern|Laden|Ende',
         1,1,darkgray,lightgray,black,menue1)
  else
   vmenu(rechts+1,oben+1,getmaxx-1,unten-1,-11, // -12
         'Neu|Stdte~ergnzen|Stdte~lschen|Entfer-~nungen|Weg|Verfahren|'
//         +tsp_aufruf+'|Serie|Drucken|Speichern|Laden|Ende',
         +tsp_aufruf+'|Serie|Speichern|Laden|Ende',
         1,1,darkgray,lightgray,black,menue1);
 end;

procedure start0;
 (* startet Stoppuhr *)
 begin
  zeit1:=now;
 end;
procedure start;
 (* 'Loesung wird berechnet'(und evtl. Serienteil) ausgeben und
     Stoppuhr starten *)
 begin
  meldung('Lsung wird berechnet',blue,white);
  if lname<>'' then outtextxy(getmaxx-11,unten+4,upcase(vname));
  start0;
 end;
procedure stop;
 (* stoppt Stoppuhr *)
 begin
  zeit2:=now;
 end;
function zeit:real;
 (* liefert gestoppte Zeit in Sek. *)
 begin
  if (zeit1=0) or (zeit2<=zeit1) then
   zeit:=0
  else
   zeit:=millisecondsbetween(zeit2,zeit1)/1000;
 end;
procedure zeit_aus(komma:boolean);
 (* gibt gestoppte Zeit an akt. Pos. aus, evtl. mit vorangestelltem Komma *)
 var s:string;
 begin
  verstecke_maus;
  if komma then outtext(', ');
  str(zeit:1:2,s); outtext('Zeit: '+s+' s');
  zeige_maus;
 end;

procedure zeichne_stadt(nr:stadtnr);
 (* zeichnet Stadt nr in Karte ein, vorher windowon noetig *)
 begin
  if nr<>0 then numdot(xv(stadtkarte[nr].x),yv(stadtkarte[nr].y),nr);
 end;
procedure zeichne_kante(stadt1,stadt2:stadtnr);
 (* zeichnet Pfeil v. stadt1 nach stadt2, vorher window noetig *)
 begin
  if (stadt1<>stadt2) and (stadt1<>0) and (stadt2<>0) then
   arrow(xv(stadtkarte[stadt1].x),yv(stadtkarte[stadt1].y),
         xv(stadtkarte[stadt2].x),yv(stadtkarte[stadt2].y));
 end;
procedure zeichne_karte(loeschen:boolean;farbe:word);
 (* zeichnet Staedte mit farbe in Karte, loeschen=true: bildschirm loeschen *)
 var i:stadtnr;
 begin
  verstecke_maus; windowon; if loeschen then clearviewport;
  setcolor(farbe);
  for i:=1 to stadtanzahl do zeichne_stadt(i);
  windowoff; zeige_maus;
 end;
procedure zeichne_weg(w:weg;loeschen:boolean;farbe:word);
 (* zeichnet Weg mit farbe in Karte, loeschen=true: bildschirm loeschen *)
 var i:byte;
 begin
  verstecke_maus; windowon; if loeschen then clearviewport;
  setcolor(farbe);
  for i:=2 to length(w) do
   zeichne_kante(ord(w[i-1]),ord(w[i]));
  windowoff; zeige_maus;
 end;
procedure farbstadt(nr:stadtnr;farbe:word);
 (* zeichnet Stadt nr mit farbe in Karte *)
 begin
  windowon; setcolor(farbe);
  zeichne_stadt(nr); windowoff;
 end;
procedure startstadt(w:weg;farbe:word);
 (* zeichnet Startstadt von w mit farbe in Karte *)
 begin
  windowon; setcolor(farbe);
  if length(aktuell)<>0 then zeichne_stadt(ord(aktuell[1]));
  windowoff;
 end;
procedure wegkarte(w:weg;loeschen:boolean;wegfarbe,kartenfarbe:word);
 (* zeichnet Weg und Karte, loeschen=true: bildschirm loeschen *)
 begin
  zeichne_weg(w,loeschen,wegfarbe);
  zeichne_karte(false,kartenfarbe);
 end;

procedure bestimme_entfernung(nr:stadtnr;typ:entftyp);
 (* traegt Entfernungen fuer Stadt nr in entftab ein, Berechnung
    entsprechend typ *)
 var i:stadtnr;
     r:real;
 begin
  for i:=1 to stadtanzahl do begin
   if typ=kart then
    entftab[nr,i]:=round(sqrt(sqr(stadtkarte[nr].x-stadtkarte[i].x)+
                         sqr(stadtkarte[nr].y-stadtkarte[i].y))*1000)
   else
    entftab[nr,i]:=trunc(random(maxzufall))*1000+1000;
   if typ=asymm then
    entftab[i,nr]:=trunc(random(maxzufall))*1000+1000
   else
    entftab[i,nr]:=entftab[nr,i];
  end;
  entftab[nr,nr]:=0;
 end;
procedure zeige_entftab(schirm:boolean);
 (* gibt entftab auf Bildschirm(true) oder in Datei(false) aus *)
 var x,y:stadtnr;
     s:string;
 procedure print(s:string);
  begin
   if schirm then boxout(s,true) else write(datei,s);
  end;
 procedure println;
  begin
   if schirm then boxcr else writeln(datei);
  end;
 begin
  boxoutexit:=false;
  if schirm then begin
   windowon; clearviewport; windowoff;
   settextbox(10,oben+10,rechts-10,unten-10,10,1,1,magenta,white);
   textbox;
  end;
  for x:=1 to stadtanzahl do begin
   if not boxoutexit then begin
    str(x:1,s); println; print('#'+s); println;
   end;
   for y:=1 to stadtanzahl do begin
    if not boxoutexit then begin
     str(entftab[x,y]/1000:9:3,s); print(' '+s);
     if (not schirm) and (y mod 7=0) then writeln(datei);
    end;
   end;
  end;
 end;

function finde_stadt(xx,yy:longint):stadtnr;
 (* findet zu (xx,yy) naechste Stadt *)
 var i,naechste_stadt:stadtnr;
     abstand,minabstand:real;
 begin
  minabstand:=1.0e38; naechste_stadt:=0;
  for i:=1 to stadtanzahl do begin
   abstand:=sqr(xx-stadtkarte[i].x)+sqr(yy-stadtkarte[i].y);
   if abstand<minabstand then begin
    minabstand:=abstand; naechste_stadt:=i;
   end;
  end;
  finde_stadt:=naechste_stadt;
 end;
function stadtwahl(button:byte):stadtnr;
 (* wartet auf 'button'(255: bel.) und bestimmt naechstgelegene Stadt *)
 begin
  warte_maus(0); zeige_maus;
  if button<255 then
   warte_maus(button)
  else
   repeat mausstatus; until maus_status<>0;
  stadtwahl:=finde_stadt(mx(mausx),my(mausy));
  verstecke_maus;
 end;
procedure neue_stadt(xx,yy:longint);
 (* traegt stadt mit Koord. (xx,yy) in stadtkarte ein und zeichnet sie *)
 begin
  if stadtanzahl<maxstadt then begin
   inc(stadtanzahl);
   stadtkarte[stadtanzahl].x:=xx; stadtkarte[stadtanzahl].y:=yy;
   if entftabok then bestimme_entfernung(stadtanzahl,entftabtyp);
   windowon; setcolor(yellow);
   numdot(xv(xx),yv(yy),stadtanzahl);
   windowoff;
  end else begin
   meldung('Maximal ',red,white); outint(maxstadt,1);
   outtext(' Stdte mglich!'); letzte_meldung:=true;
  end
 end;
procedure loesche_stadt(nr:stadtnr);
 (* entfernt Stadt nr aus stadtkarte, Wege werden geloescht *)
 var i,j:stadtnr;
 begin
  for i:=nr to stadtanzahl-1 do begin
   stadtkarte[i]:=stadtkarte[i+1];
   entftab[i]:=entftab[i+1];
  end;
  for i:=1 to stadtanzahl-1 do
   for j:=nr to stadtanzahl-1 do entftab[i,j]:=entftab[i,j+1];
  dec(stadtanzahl); vorher:=''; aktuell:='';
  if entftabok then entftabok:=stadtanzahl<>0;
 end;
procedure nrweg(var w:weg);
 (* erzeugt Rundweg nach Staedtnr. *)
 var i:stadtnr;
 begin
  w:='';
  for i:=1 to stadtanzahl do w:=w+chr(i);
  w:=w+chr(1);
 end;
procedure zufallsweg(var w:weg);
 (* erzeugt zufaelligen Rundweg *)
 var i,j:stadtnr;
     hilf:char;
 begin
  w:='';
  for i:=1 to stadtanzahl do w:=w+chr(i);
  for i:=1 to stadtanzahl do begin
   j:=random(stadtanzahl)+1;
   hilf:=w[i]; w[i]:=w[j]; w[j]:=hilf;
  end;
  w:=w+w[1];
 end;
procedure manuellweg(var w:weg);
 (* manuelle Wegeingabe *)
 var nr:stadtnr;
     fertig:boolean;
 procedure maxwarnung;
  begin
   windowoff;
   meldung('Maximal 255 Wegpunkte mglich!',red,white);
   warte_maus(0); zeige_maus; warte; verstecke_maus;
   menue(-3,'neuer Weg|Unterbrechung|fertig',green,white,black,menue2);
   windowon;
  end;
 begin
  menue(-3,'neuer Weg|Unterbrechung|fertig',green,white,black,menue2);
  zeige_maus; windowon;
  fertig:=false;
  repeat
   windowon;
   if w<>'' then begin setcolor(cyan); zeichne_stadt(ord(w[length(w)])); end;
   zeige_maus; warte_maus(0);
   repeat mausstatus; until maus_status in [1,2];
   verstecke_maus;
   if maus_status=1 then
    case chkmenu(mausx,mausy,menue2) of
     0: begin
         if length(w)<255 then begin
          nr:=finde_stadt(mx(mausx),my(mausy)); w:=w+chr(nr);
          if length(w)>1 then
           if ord(w[length(w)-1])<>nr then begin
            setcolor(brown); zeichne_kante(ord(w[length(w)-1]),nr);
            setcolor(yellow); zeichne_stadt(ord(w[length(w)-1]));
           end;
          setcolor(cyan); zeichne_stadt(nr);
         end else
          maxwarnung;
        end;
     1: begin
         w:=''; zeichne_karte(true,yellow);
        end;
     2: begin
         if w<>'' then
          if length(w)<255 then begin
           if w[length(w)]<>chr(0) then begin
            setcolor(yellow); zeichne_stadt(ord(w[length(w)]));
            w:=w+chr(0);
           end
          end else
           maxwarnung;
        end;
     3: fertig:=true;
    end
   else begin
    nr:=finde_stadt(mx(mausx),my(mausy));
    if w<>'' then begin
     w:=copy(w,1,length(w)-1);
     zeichne_weg(w,true,brown); zeichne_karte(false,yellow);
    end;
   end;
  until fertig;
  windowoff;
 end;

function dateiname(prompt:string):string;
 (* liefert Dateiname, bei $ wird Directory angezeigt *)
 var name,dname:string;
     p:byte;
     dummy:boolean;
 begin
  name:='$';
  verstecke_maus;
  repeat
   meldung(prompt,green,white); ginput(name,79-length(prompt),green,white);
   if (copy(name,1,2)='b:') or (copy(name,1,2)='B:') then name[1]:='a';
   p:=pos('$',name);
   if p<>0 then begin
    windowon; clearviewport; windowoff;
    settextbox(10,oben+10,rechts-10,unten-10,10,1,1,magenta,white);
    textbox; meldung('(Maus-)Taste drcken',green,white);
    dname:=name; if p=length(dname) then dname:=dname+'*.tsp';
    delete(dname,p,1); dummy:=gdir(dname);
   end else
    if (name<>'') and (pos('.',name)=0) then name:=name+'.tsp';
  until p=0;
  zeige_maus;
  dateiname:=name;
 end;
procedure speichern(name:string);
 (* als ASCII-Datei, kann mit bel. Editor bearbeitet werden *)
 var i:word;
     j:stadtnr;
 procedure wegaus(w:weg);
  var i:byte;
  begin
{$I-}
   for i:=1 to length(w) do begin
    write(datei,ord(w[i]):4);
    if (i mod 19)=0 then writeln(datei);
   end;
   writeln(datei,-1:4);
  end;
 begin
  if name<>'' then begin
   assign(datei,name);
   i:=ioresult; rewrite(datei);
   if ioresult=0 then begin
    write(datei,'#TSP V',version:1:2);
    if zeit1=0 then writeln(datei)
               else writeln(datei,', letzte Rechenzeit: ',zeit:1:2,' s');
    writeln(datei); writeln(datei,'#Staedteanzahl');
    writeln(datei,stadtanzahl); writeln(datei);
    if not entftabok then begin
     entftabtyp:=kart;
     for j:=1 to stadtanzahl do
      bestimme_entfernung(j,entftabtyp);
     entftabok:=true; laenge_aus(aktuell);
    end;
    writeln(datei,'#Art der Entfernungsberechnung(0: aus Karte, 1: symmetrisch, 2: asymmetrisch)');
    writeln(datei,ord(entftabtyp));
    writeln(datei); write(datei,'#Entfernungstabelle');
    zeige_entftab(false); writeln(datei);
    writeln(datei);
    writeln(datei,'#vorheriger Weg(-1: Ende), Laenge: ',weglaenge(vorher)/1000:1:3);
    wegaus(vorher);
    writeln(datei,'#aktueller Weg(-1: Ende), Laenge: ',weglaenge(aktuell)/1000:1:3);
    wegaus(aktuell);
    writeln(datei);
    writeln(datei,'#Staedtekoordinaten(Maxx: ',maxx:1,', Maxy: ',maxy:1,')');
    for j:=1 to stadtanzahl do
     writeln(datei,stadtkarte[j].x:4,stadtkarte[j].y:4);
   end;
   close(datei);
   if ioresult<>0 then meldung('Fehler beim Speichern!',red,white);
{$I+}
  end;
 end;
procedure laden(name:string);
 (* Nichtzahlen, Zeilenenden und mit '#' beginnende Zeilen werden ueberlesen *)
 var i:word;
     j,k:stadtnr;
     fehler:boolean;
 function zahl(min,max:longint;faktor:real):longint;
  (* liest naechste (Real-)Zahl und multipliziert sie mit faktor,
     fehler:=true und zahl:=min, falls ausserhalb (min..max) *)
  var c:char;
      s:string;
      r:real;
      dummy:integer;
  begin
   if fehler then
    zahl:=min
   else begin
    c:=' '; s:='';
{$I-}
    while not(eof(datei) or (c in ['0'..'9','-','.'])) do begin
     read(datei,c); if c='#' then readln(datei,s);
    end;
    if c='.' then s:='0.' else s:=c;
    if c='-' then c:='0';
    while (not eof(datei)) and (c in ['0'..'9','.']) do begin
     read(datei,c); s:=s+c;
    end;
{$I+}
    if s[length(s)-1]='.' then insert('0',s,length(s));
    val(copy(s,1,length(s)-1),r,dummy); r:=r*faktor;
    if (r<min) or (r>max) or (ioresult<>0) then begin
     fehler:=true; zahl:=min;
    end else
     zahl:=round(r);
   end;
  end;
 procedure wegein(var w:weg);
  var nr:integer;
  begin
   w:='';
   repeat
    nr:=zahl(-1,maxstadt,1);
    if nr>=0 then begin
     if length(w)<255 then w:=w+chr(nr) else fehler:=true;
    end;
   until nr<0;
  end;
 begin
  if name<>'' then begin
   fehler:=false;
{$I-}
   assign(datei,name);
   i:=ioresult; reset(datei);
   if ioresult=0 then begin
    neustart;
    stadtanzahl:=zahl(1,maxstadt,1);
    case zahl(0,2,1) of
     0: entftabtyp:=kart;
     1: entftabtyp:=symm;
     2: entftabtyp:=asymm;
    end;
    for j:=1 to stadtanzahl do
     for k:=1 to stadtanzahl do
      entftab[j,k]:=zahl(0,maxstrecke-1,1000);
    entftabok:=true;
    wegein(vorher);
    wegein(aktuell);
    for j:=1 to stadtanzahl do begin
     stadtkarte[j].x:=zahl(0,maxx,1);
     stadtkarte[j].y:=zahl(0,maxy,1);
    end;
   end;
   close(datei);
   if (ioresult<>0) or fehler then begin
    neustart; meldung('Fehler beim Laden!',red,white);
   end else
    laenge_aus(aktuell);
 {$I+}
  end;
  verstecke_maus; if name='' then laenge_aus(aktuell);
  wegkarte(aktuell,true,brown,yellow); zeige_maus;
 end;
function serienteil(wegtypen:wegtypmenge):boolean;
 (* letzte Serienberechnung speichern, naechste laden,
    false: keine weitere Datei oder falscher Wegtyp *)
 var name:string;
     p:byte;
     eintrag:searchrec;
 begin
  if vname<>'/' then begin
   name:=sname; p:=pos('*',name); if p<>0 then name[p]:=vname;
   speichern(name);
  end;
  if (vname='0') and (pos('*',lname)=0) then lname:='';
  if lname<>'' then begin
   repeat
    if vname='9' then vname:='a' else inc(vname);
    name:=sname; p:=pos('*',name); if p<>0 then name[p]:=vname;
    findfirst(name,anyfile,eintrag);
   until doserror<>0;
   name:=lname; p:=pos('*',name);
   if p<>0 then name[p]:=vname;
   findfirst(name,anyfile,eintrag);
   if doserror=0 then laden(name) else lname:='';
  end;
  if lname='' then serienteil:=false
              else serienteil:=loesen(wegtypen);
 end;

procedure eine_stadt;
 (* Mausklick auf Karte -> eine Stadt ergaenzen/loeschen *)
 begin
  verstecke_maus;
  if maus_status=1 then begin
    neue_stadt(mx(mausx),my(mausy));
    if not letzte_meldung then laenge_aus(aktuell);
  end else
   if stadtanzahl>0 then begin
    loesche_stadt(finde_stadt(mx(mausx),my(mausy)));
    zeichne_karte(true,yellow);
    laenge_aus(aktuell);
   end;
  zeige_maus;
 end;
procedure neu;
 (* alles loeschen *)
 begin
  verstecke_maus;
  menue(-3,'Alles lschen?|Ja|Nein',green,white,black,menue2);
  zeige_maus; warte_maus(0); warte_maus(1); verstecke_maus;
  if chkmenu(mausx,mausy,menue2)=2 then begin
   neustart;
   box(1,oben+1,rechts-1,unten-1,black);
  end;
  laenge_aus(aktuell); zeige_maus;
 end;
procedure staedte_ergaenzen;
 (* mehrere Staedte ergaenzen *)
 var nstr:string;
     n,i,dummy:integer;
 begin
  verstecke_maus;
  if stadtanzahl<maxstadt then begin
   meldung('Wie viele neue Stdte? ',green,white);
   str(maxstadt-stadtanzahl:1,nstr); ginput(nstr,4,green,white);
   laenge_aus(aktuell);
   val(nstr,n,dummy); if stadtanzahl+n>maxstadt then n:=maxstadt-stadtanzahl+1;
  end else
   n:=1;
  for i:=1 to n do
   neue_stadt(random(maxx+1),random(maxy+1));
  zeige_maus;
 end;
procedure staedte_loeschen;
 (* mehrere Staedte loeschen *)
 var start,ende,i,j:stadtnr;
 begin
  if stadtanzahl=0 then
   meldung('Keine Stadt vorhanden!',red,white)
  else begin
   meldung('Erste und letzte zu lschende Stadt auswhlen!',green,white);
   warte_maus(0); warte_maus(1); start:=finde_stadt(mx(mausx),my(mausy));
   setcolor(cyan); windowon; verstecke_maus;
   numdot(xv(stadtkarte[start].x),yv(stadtkarte[start].y),start);
   zeige_maus;
   warte_maus(0); warte_maus(1); ende:=finde_stadt(mx(mausx),my(mausy));
   if ende<start then begin
    i:=start; start:=ende; ende:=i;
   end;
   verstecke_maus;
   for i:=start to ende do
    numdot(xv(stadtkarte[i].x),yv(stadtkarte[i].y),i);
   windowoff;
   menue(-3,'Wirklich lschen?|Ja|Nein',green,white,black,menue2);
   zeige_maus; warte_maus(0); warte_maus(1); verstecke_maus;
   if chkmenu(mausx,mausy,menue2)=2 then begin
    for i:=1 to stadtanzahl-ende do begin
     stadtkarte[start+i-1]:=stadtkarte[ende+i];
     entftab[start+i-1]:=entftab[ende+i];
    end;
    for i:=1 to stadtanzahl-ende+start-1 do
     for j:=1 to stadtanzahl-ende do entftab[i,start+j-1]:=entftab[i,ende+j];
    dec(stadtanzahl,ende-start+1); vorher:=''; aktuell:='';
    if entftabok then entftabok:=stadtanzahl<>0;
   end;
   zeichne_weg(aktuell,true,brown); zeichne_karte(false,yellow);
   laenge_aus(aktuell); zeige_maus;
  end;
 end;
procedure entfernungen;
 (* Entfernungen berechnen/eingeben *)
 var m:byte;
     i:stadtnr;
 begin
  verstecke_maus;
  if stadtanzahl=0 then
   meldung('Keine Stadt vorhanden!',red,white)
  else begin
   menue(-4,'aus Karte|symmetrisch|asymmetrisch|anzeigen',
         green,white,black,menue2);
   zeige_maus; warte_maus(0); warte_maus(1); verstecke_maus;
   m:=chkmenu(mausx,mausy,menue2);
   if m=0 then
    laenge_aus(aktuell)
   else begin
    case m of
     1 : entftabtyp:=kart;
     2 : entftabtyp:=symm;
     3 : entftabtyp:=asymm;
    end;
    if m=4 then begin
     if entftabok then begin
      meldung('(Maus-)Taste drcken',green,white);
      zeige_entftab(true);
      if not boxoutexit then begin
       zeige_maus; warte_maus(0); warte; verstecke_maus;
      end;
      windowon; zeichne_weg(aktuell,true,brown); zeichne_karte(false,yellow);
      laenge_aus(aktuell);
     end else
      meldung('Keine Entfernungstabelle vorhanden!',red,white);
    end else begin
     for i:=1 to stadtanzahl do
      bestimme_entfernung(i,entftabtyp);
     entftabok:=true; laenge_aus(aktuell);
    end;
   end;
  end;
  zeige_maus;
 end;
procedure weg_bestimmen;
 (* auf verschiedene Arten Weg vorgeben *)
 var m:byte;
     w:weg;
 begin
  verstecke_maus;
  if stadtanzahl=0 then
   meldung('Keine Stadt vorhanden!',red,white)
  else begin
   if vorher<>'' then
    menue(-5,'berlagerung|vorheriger|manuell|nach Nummern|zufllig',
         green,white,black,menue2)
   else
    menue(-3,'manuell|nach Nummern|zufllig',
         green,white,black,menue2);
   zeige_maus; warte_maus(0); warte_maus(1); verstecke_maus;
   m:=chkmenu(mausx,mausy,menue2); if (vorher='') and (m<>0) then inc(m,2);
   case m of
    1: begin
        zeichne_weg(vorher,true,darkgray);
        zeichne_weg(aktuell,false,brown);
        zeichne_karte(false,yellow);
        meldung('(Maus-)Taste drcken',green,white);
        zeige_maus; warte0; verstecke_maus;
       end;
    2: begin
        w:=aktuell; aktuell:=vorher; vorher:=w;
       end;
    3: begin
        vorher:=aktuell; manuellweg(aktuell);
       end;
    4: begin
        vorher:=aktuell; nrweg(aktuell);
       end;
    5: begin
        vorher:=aktuell; zufallsweg(aktuell);
       end;
   end;
   zeichne_weg(aktuell,true,brown); zeichne_karte(false,yellow);
   laenge_aus(aktuell);
  end;
  zeige_maus;
 end;
procedure verfahren_waehlen(var wegtypen:wegtypmenge);
 (* verfahren:=nr Lsungsverfahren, wegtypen:=erlaubte Wegtypen des Verf.
    Format verftext: '*Menueeintrag|Titelzeile',
    *=+: fragen, ob automatische Startpunktwahl gewuenscht *)
 var i:byte;
 begin
  verstecke_maus;
  windowon; clearviewport; windowoff;
  rmenu(10,oben+10,rechts-10,unten-10,-verfformat.x,-verfformat.y
        ,1,magenta,black,menue2);
  meldung('Lsungsverfahren whlen!',green,white);
  for i:=1 to verfanzahl do
   rmenuentry(i,copy(verftext[i],2,pos('|',verftext[i])-2),1,white,menue2);
  zeige_maus; warte_maus(0); warte_maus(1); verstecke_maus;
  i:=chkmenu(mausx,mausy,menue2);
  if (i>0) and (i<=verfanzahl) then begin
   verfahren:=i; wegtypen:=verfwegtypen[verfahren];
   tsp_titel:=copy(verftext[verfahren],pos('|',verftext[verfahren])+1,255);
   textbar(1,1,getmaxx-15,oben-1,tsp_titel,1,1,blue,white);
   if verftext[verfahren][1]='+' then begin
    menue(-3,'Automat. Startpunktwahl?|ja|nein',green,white,black,menue2);
    zeige_maus; warte_maus(0); warte_maus(1); verstecke_maus;
    startpunktwahl:=chkmenu(mausx,mausy,menue2)=2;
   end else
    startpunktwahl:=true;
  end;
  wegkarte(aktuell,true,brown,yellow); laenge_aus(aktuell);
  zeige_maus;
 end;
function loesen(wegtypen:wegtypmenge):boolean;
 (* true: wegtyp(aktuell) in Menge wegtypen enthalten und entftab
    vorhanden, sonst Meldung *)
 begin
  if (verfanzahl<>0) and (verfahren=0) then begin
   loesen:=false; meldung('Kein Verfahren gewhlt!',red,white);
  end else
   if wegtyp(aktuell) in wegtypen then begin
     loesen:=entftabok;
     if not entftabok then
      meldung('Keine Entfernungstabelle vorhanden!',red,white);
   end else begin
    loesen:=false; meldung('Falscher Wegtyp!',red,white);
   end;
 end;
procedure serie;
 (* Load-/Save-Name fuer Serienberechnung *)
 begin
  if (verfanzahl<>0) and (verfahren=0) then
   meldung('Kein Verfahren gewhlt!',red,white)
  else begin
   lname:=dateiname('Name beim Laden(*=var)? ');
   if lname<>'' then sname:=dateiname('Name beim Speichern(*=var)? ');
   if sname='' then lname:='';
   vname:='/';
   wegkarte(aktuell,true,brown,yellow); laenge_aus(aktuell);
  end;
 end;
(*
procedure drucken;
 (* Karte und Weglaenge drucken *)
 var typ:byte;
 begin
  if stadtanzahl=0 then
   meldung('Nichts zum Drucken!',red,white)
  else begin
   menue(-3,'Welcher Druckertyp?|Epson|Hewlett-Packard'
         ,green,white,black,menue2);
   warte_maus(0); warte_maus(1); typ:=chkmenu(mausx,mausy,menue2);
   verstecke_maus; laenge_aus(aktuell);
{$I-}
   if typ>1 then begin
    writeln(lst,tsp_titel); writeln(lst);
    if hrdcpy(0,oben,rechts,unten,typ-2)=0 then begin
     if entftabok then begin
      writeln(lst);
      writeln(lst,'Lnge des Weges: ',weglaenge(aktuell)/1000:1:3);
      write(lst,#12);
     end;
{$I+}
    end else
     meldung('Drucker nicht bereit!',red,white);
   end;
   zeige_maus;
  end;
 end;
*)
procedure datei_speichern;
 (* Name waehlen und speichern *)
 var name:string;
 begin
  if stadtanzahl=0 then
   meldung('Nichts zum Speichern!',red,white)
  else begin
   name:=dateiname('Dateiname($=dir)? '); laenge_aus(aktuell);
   speichern(name);
   verstecke_maus; wegkarte(aktuell,true,brown,yellow); zeige_maus;
  end;
 end;
procedure datei_laden;
 (* Name waehlen und laden *)
 var name:string;
 begin
  name:=dateiname('Dateiname($=dir)? ');
  laden(name);
 end;
function ende:boolean;
 (* Sicherheitsabfrage *)
 begin
  verstecke_maus;
  menue(-3,'Wirklich beenden?|Ja|Nein',green,white,black,menue2);
  zeige_maus; warte_maus(0); warte_maus(1); verstecke_maus;
  if chkmenu(mausx,mausy,menue2)=2 then
   ende:=true
  else begin
   laenge_aus(aktuell); zeige_maus;
   ende:=false;
  end;
 end;

function tspmenue(wegtypen:wegtypmenge):boolean;
 (* ueberwacht Menue, Verlassen mit Ende/Schliessbutton(false) oder
    Loesen(true, nur wenn aktueller Weg in wegtypen-Menge *)
 var aufruf,enden:boolean;
     wahl:word;
 begin
  if verfanzahl=0 then tsp_wegtypen:=wegtypen;
  warte_maus(0); zeige_maus;
  repeat
   if lname<>'' then begin
    aufruf:=serienteil(tsp_wegtypen); enden:=false;
   end else begin
    repeat mausstatus; until maus_status in [1,2];
    aufruf:=false; wahl:=chkmenu(mausx,mausy,menue1);
    if (chkmenu(mausx,mausy,button)=1) or (wahl=menue1.anzahl) then begin
     enden:=ende; warte_maus(0);
    end else begin
     if (verfanzahl=0) and (wahl>=6) then inc(wahl);
     case wahl of
      0 : begin
           if chkmenu(mausx,mausy,karte)=1 then eine_stadt;
           if chkmenu(mausx,mausy,leiste)=1 then
            if maus_status=1 then laenge_aus(aktuell)
                             else wegtyp_aus(aktuell);
          end;
      1 : neu;
      2 : staedte_ergaenzen;
      3 : staedte_loeschen;
      4 : entfernungen;
      5 : weg_bestimmen;
      6 : verfahren_waehlen(tsp_wegtypen);
      7 : aufruf:=loesen(tsp_wegtypen);
      8 : serie;
//      9 : drucken;
      9 : datei_speichern;
      10: datei_laden;
     end;
     enden:=false; warte_maus(0);
    end;
   end
  until aufruf or enden;
  vorher:=aktuell; tspmenue:=not enden;
 end;

function tspinit(titel,aufruf:string):boolean;
 (* Initialisierungen, Bildschirm zeichnen
    titel: Fenstertitel, aufruf: Menuetext Loesungsroutine
    false: kein Maustreiber installiert *)
 begin
  hireson; mauseinsatz; verstecke_maus;
  if mausok then begin
   randomize; setarrowstyle(8,20,0.5);
   tsp_titel:=titel; tsp_aufruf:=aufruf;
   oben:=14; rechts:=getmaxx-79; unten:=getmaxy-14;
   neustart; lname:='';
   bildschirm;
   meldung('TSP-Unit V',blue,white); outreal(version,1,2);
   outtext(' - (c) 2013 Thomas Klein');
   letzte_meldung:=true;
   rect(1,unten+1,getmaxx-1,getmaxy-1,leiste);
   setwindow(1,oben+1,rechts-1,unten-1,maxx,maxy); windowoff;
   rect(1,oben+1,rechts-1,unten-1,karte);
  end else begin
   closegraph; writeln('Kein Maustreiber installiert!'); writeln;
  end;
  tspinit:=mausok;
 end;

begin
 verfahren:=0; verfanzahl:=0; verfformat.x:=5; verfformat.y:=8;
end.

